
#ifdef WIN32
#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#endif
#include <GL/gl.h>	
#include <GL/glu.h>	
#include <stdlib.h>
#include <time.h>
#include <SDL.h>
#include <QImage.h>
#include <QMessageBox.h>
#include <qapplication.h>
#include <QGL.h>
#include <QDom.h>
#include <QFile.h>
#include <QDir.h>

#include <vector>
#include <algorithm>

#include <vcg/Point3.h>
#include <vcg/Matrix44.h>
#include "CISG.h"
#include "CIMoebius.h"
#include "CIMd2.h"

using namespace vcg;
using namespace std;
int msx=0,msy=0;

class CISGM : public CISG
{
public :
	virtual CISGNode *Allocate(const QString &classname)
	{
		CISGNode *ret=CISG::Allocate(classname);		
		if(classname=="CIMoebius") ret=new CIMoebius();
		
		return ret;
	}
	
};

CISGM scene;
CIMd2 model;

void DrawGLScene()
{
		long hits;	
		static unsigned int selectBuf[16384];
	
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	float spec[4]={.4f,.4f,.4f,1};
	float shin = 1;
	glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR , spec);
	glMaterialf(GL_FRONT_AND_BACK,GL_SHININESS , shin);
	
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE );
	glEnable(GL_COLOR_MATERIAL);
	
	if(msx && msy)
	{
		glSelectBuffer(16384, selectBuf); 
		glRenderMode(GL_SELECT);
		glInitNames();  glPushName(-1);
		int viewport[4];  glGetIntegerv(GL_VIEWPORT,          viewport);
		double   mp[16];  glGetDoublev( GL_PROJECTION_MATRIX ,mp);
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
		gluPickMatrix(msx, viewport[3]-msy, 4, 4, viewport);
		glMultMatrixd(mp);
		glMatrixMode(GL_MODELVIEW);
  }
	float lightpos[4]={0,0,1,0};
	glLoadIdentity();
	glLightfv(GL_LIGHT0,GL_POSITION,lightpos);
	gluLookAt(2,2,12,0,0,0,0,1,0);

  glColor3f(1,.4f,.4f);
	glPushMatrix();
	glScalef(.04f,.04f,.04f);
	glRotatef(-180,0,1,0);
	glRotatef(-90,1,0,0);
	glLoadName(1);
	model.DrawTimedFrame(clock()/1000.0f);
  glPopMatrix();
	glLoadName(2);
	scene.glDraw(clock()/1000.0f);	
	if(msx && msy)
	{
		glPopMatrix();
  glMatrixMode(GL_PROJECTION);
  glPopMatrix();
  glMatrixMode(GL_MODELVIEW);
  hits = glRenderMode(GL_RENDER);
	if (hits > 0)   {
  vector< pair<double,unsigned int> > H;
  for(int ii=0;ii<hits;ii++)
    H.push_back( make_pair(selectBuf[ii*4+1]/4294967295.0,selectBuf[ii*4+3]));
  sort(H.begin(),H.end());
  printf("nearest is %i \n",H[0].second);
	}
	msx=msy=0;
	}
	SDL_GL_SwapBuffers();
}

// Inizializzazione dello stato di OpenGL
void myGLInit()
{
	// Abilita il la possibilita' di cambiare il alcune componenti colore del 
	// materiale tramite un semplice glcolor, anzich fare glMaterial(...)
	glColorMaterial(GL_FRONT_AND_BACK,GL_AMBIENT_AND_DIFFUSE);
	glEnable(GL_COLOR_MATERIAL);

	// Abilita il calcolo dell'illuminazione
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);

	// anche le normali attraversano la pipeline, quindi devono essere  
	// rinormalizzate, prima del calcolo dell'illuminazione, altrimenti 
	// il calcolo dell'illuminazione viene sbagliato.
	glEnable(GL_NORMALIZE); 
	
	// Abilita il test sullo zbuffer e quindi l'eliminazione delle 
	// superfici nascoste
	glEnable(GL_DEPTH_TEST); 	

	// Disabilita lo shading di gouraud per le facce dei poligoni disegnati
	glShadeModel(GL_SMOOTH);

	// abilita i texture environment necessari
	glEnable(GL_TEXTURE_2D);
	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);


}

void myGLReshapeFunc(GLsizei w, GLsizei h)
{
	glMatrixMode (GL_PROJECTION);   
	glLoadIdentity ();
	float ratio=(float)w/(float)h;
  gluPerspective(45,ratio,3,20);
	glViewport (0, 0, (GLsizei) w, (GLsizei) h); 
	glMatrixMode (GL_MODELVIEW);
	myGLInit();
}


int main(int argc, char **argv) 
{  
	// Loading del modello quake2 e settaggio dell'animazione
	model.Load("data/terminator/tris.md2","data/terminator/dwing.png");
	model.SetAnim(0,40);    // Range dei frame dell'animazione di interesse
	model.SetFPS(2.0f);        // velocita del playback dell'animazione
	
  /* Initialize SDL for video output */
  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
    fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
    exit(1);
  }
  /* Specify the requirements for the gl window*/
	SDL_GL_SetAttribute(SDL_GL_DEPTH_SIZE, 24);   
  
	/* Create a 640x480 OpenGL screen */
  if ( SDL_SetVideoMode(640, 480, 0, SDL_OPENGL |SDL_RESIZABLE) == NULL ) {
    fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
    SDL_Quit();
    exit(2);
  }

	//scene.root.Sons.back()->Sons.push_back(new CISGTransformation));
//	scene.root.Sons.back()->Sons.push_back(new CISGTransformation));

 /****  Creazione dello scenegraph*****/

 // Gruppo dell'anello di moebius
 CISGGroup *mg = new CISGGroup();
 scene.root.Sons.push_back(mg);
 
 // creazione dell'anello di moebius
 CIMoebius *ms=new CIMoebius();  
 CIMoebius::myShape s;  s.BuildNGon(3,1);
 ms->GenerateMoebiusRing(s,36,3,120);
 // aggiunta dei nodi allo sg.

 mg->Sons.push_back(new CISGTranslation(Point3f(0,3,5)));
 mg->Sons.push_back(new CISGScaling(.3f));
 mg->Sons.push_back(new CISGRotation(90,Point3f(1,0,0)));
 mg->Sons.push_back(new CISGAnimZPrecession());
 mg->Sons.push_back(ms);

 // Aggiunta del terreno
 scene.root.Sons.push_back(new CISGRotation(90,Point3f(1,0,0)));
 scene.root.Sons.push_back(new CISGScaling(3));
 scene.root.Sons.push_back(new CISGGround());

 QFile xf("test.xml");
 xf.open(IO_WriteOnly);
 QTextStream xstrm(&xf);
 scene.XMLWrite(xstrm);
 xf.close();
//scene.XMLRead("test.xml");
 
 /* Loop, drawing and checking events */
	int done = 0;
	int ret;
  myGLReshapeFunc(640,480);  
	while ( ! done ) 
	{
    SDL_Event event;
	  ret=SDL_PollEvent(&event);
		if(ret)
			switch(event.type)
			{  
						case SDL_QUIT    : done = 1;  
															break ;
						case SDL_KEYDOWN : if ( event.key.keysym.sym == SDLK_ESCAPE ) 
																				done = 1;
															 if ( event.key.keysym.sym == SDLK_SPACE ) 
																					QMessageBox::information(0,"Test","test");
	
          
															break;
						case SDL_MOUSEBUTTONDOWN : msx= event.button.x; msy= event.button.y; 
												printf("mouse x%i y%i\n",msx,msy);

															break;
						case SDL_VIDEORESIZE	: //SDL_SetVideoMode(event.resize.w,event.resize.h, 0, SDL_OPENGL | SDL_RESIZABLE);
																		myGLReshapeFunc(event.resize.w,event.resize.h); 
																		break;
			}  
		else DrawGLScene();
  }
  SDL_Quit();
  return 1;
}
